using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MEdiaSpaceCanmore
{
    internal sealed partial class LargeForm : CustomForm
    {
        public LargeForm()
        {
            InitializeComponent();
        }


        // 
        // UI component handlers
        //
        private void LargeForm_Load(object sender, EventArgs e)
        {
            this.FormMinimized += new FormMinimizedEventHandler(LargeForm_FormMinimized);
            this.FormClosed += new FormClosedEventHandler(LargeForm_FormClosed);
        }

        private bool LargeForm_FormMinimized(object sender, EventArgs e)
        {
            FormController.SwitchToMedium();
            return false;
        }

        private void LargeForm_FormClosed(object sender, EventArgs e)
        {
            FormController.Shutdown();
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            FormController.OnConnectClicked();
        }

        private void doorButton_Click(object sender, EventArgs e)
        {
            FormController.OnDoorClicked();
        }

        private void chatInputBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                FormController.OnMessageEntered(chatInputBox.Text);
                chatInputBox.Text = "";
            }
        }

        private void documentButton_Click(object sender, EventArgs e)
        {
            FormController.OnDocumentClicked();
        }

        private void phoneButton_Click(object sender, EventArgs e)
        {
            FormController.OnPhoneClicked();
        }

        private void toolsButton_Click(object sender, EventArgs e)
        {
            FormController.OnToolsClicked();
        }


        //
        // Content display functions
        //
        public void SetConnected(bool connected)
        {
            if (connected)
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.disconnect;
            else
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.connect;
        }

        public void setDoorEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(doorButton, enabled);
        }

        public void SetUIEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(documentButton, enabled);
            EnableControl(toolsButton, enabled);
        }

        public void SetCallDisabled(bool disabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(phoneButton, !disabled);
        }

        public void SetCallStatus(bool talking)
        {
            if (talking)
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.redphone;
            else
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.greenphone;
        }

        public void SetMotionDetection(int value, DateTime timestamp)
        {
        }

        public void SetRemoteImage(Image image)
        {
            DisplayImageInPictureBox(incomingPictureBox, image);
        }

        public void SetLocalImage(Image image)
        {
            if (null != image)
                image.RotateFlip(RotateFlipType.Rotate180FlipY);

            DisplayImageInPictureBox(outgoingPictureBox, image);
        }

        public void ClearChatSpool()
        {
            DisplayTextInTextBox(chatOutputBox, "");
        }

        public void AddChatMessage(String sender, String message)
        {
            DisplayTextInTextBox(chatOutputBox, chatOutputBox.Text + sender + ": " + message + System.Environment.NewLine);
        }
   }
}