using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MEdiaSpaceCanmore
{
    internal sealed partial class MediumForm : CustomForm
    {
        //
        // Lifecycle
        //
        public MediumForm()
        {
            InitializeComponent();

            FormController.Initialize(this);
        }


        //
        // UI component handlers
        //
        private void MediumForm_Load(object sender, EventArgs e)
        {
            this.FormMinimized += new FormMinimizedEventHandler(MediumForm_FormMinimized);
            this.FormMaximized += new FormMaximizedEventHandler(MediumForm_FormMaximized);
            this.FormClosed += new FormClosedEventHandler(MediumForm_FormClosed);
        }

        private bool MediumForm_FormMinimized(object sender, EventArgs e)
        {
            FormController.SwitchToSmall();
            return false;
        }

        private bool MediumForm_FormMaximized(object sender, EventArgs e)
        {
            FormController.SwitchToLarge();
            return false;
        }

        private void MediumForm_FormClosed(object sender, EventArgs e)
        {
            FormController.Shutdown();
        }


        private void connectButton_Click(object sender, EventArgs e)
        {
            FormController.OnConnectClicked();
        }

        private void doorButton_Click(object sender, EventArgs e)
        {
            FormController.OnDoorClicked();
        }

        private void phoneButton_Click(object sender, EventArgs e)
        {
            FormController.OnPhoneClicked();
        }

        private void toolsButton_Click(object sender, EventArgs e)
        {
            FormController.OnToolsClicked();
        }


        //
        // Content display handlers
        //
        public void SetConnected(bool connected)
        {
            if (connected)
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.disconnect_small;
            else
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.connect_small;
        }

        public void SetDoorEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(doorButton, enabled);
        }

        public void SetUIEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(toolsButton, enabled);
        }

        public void SetCallDisabled(bool disabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(phoneButton, !disabled);
        }

        public void SetCallStatus(bool talking)
        {
            if (talking)
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.redphone_small;
            else
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.greenphone_small;
        }

        public void SetMotionDetection(int value, DateTime timestamp)
        {
        }

        public void SetRemoteImage(Image image)
        {
            DisplayImageInPictureBox(incomingPictureBox, image);
        }

        public void SetLocalImage(Image image)
        {
            if (image != null)
                image.RotateFlip(RotateFlipType.Rotate180FlipY);

            DisplayImageInPictureBox(outgoingPictureBox, image);
        }

        public void ClearChatSpool()
        {
            DisplayTextInTextBox(chatOutputBox, "");
        }

        public void AddChatMessage(String sender, String message)
        {
            DisplayTextInTextBox(chatOutputBox, chatOutputBox.Text + sender + ": " + message + System.Environment.NewLine);
        }
    }
}