using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Threading;
using System.Text;
using SKYPE4COMLib;

namespace MEdiaSpaceCore
{
    public class AudioWrapper
    {
        // Callback bits so the hosting UI can get notification about call status changes
        public delegate void AudioStatusChangedHandler(bool talking);
        public event AudioStatusChangedHandler AudioStatusChanged;

        // Callback bits to prevent multiple simultaneous outgoing calls
        public delegate void CallAttachingHandler(bool callAttaching);
        public event CallAttachingHandler CallAttaching;

        // Internal variables
        private String _sinkUsername;
        private Call _skypeCall;

        private static Skype _skypeObject = new SkypeClass();

        public AudioWrapper(String sinkUsername)
        {
            // Set up the locals
            _skypeCall = null;
            _sinkUsername = sinkUsername;
            
            // Start the client, if necessary
            // (better to do this here than waiting until we're trying to call it...)
            this.Reconnect();

            // Set up a callback to pick up call status changes
            _skypeObject.CallStatus += new _ISkypeEvents_CallStatusEventHandler(_skypeObject_CallStatus);

            // Set up a callback to monitor for user status changes, too
            _skypeObject.UserStatus += new _ISkypeEvents_UserStatusEventHandler(_skypeObject_UserStatus);
        }

        public String CurrentHandle()
        {
            if (_skypeObject != null)
                return _skypeObject.CurrentUserHandle;
            else
                return null;
        }

        public void SetSinkUsername(String sinkUsername)
        {
            _sinkUsername = sinkUsername;
        }

        public void StartCall()
        {
            if (_sinkUsername == null)
                return;

            // Double-check to make sure the client's still up and running
            this.Reconnect();

            // PlaceCall fails if we're not shown to be online
            if (_skypeObject.CurrentUserStatus == TUserStatus.cusOffline)
                _skypeObject.ChangeUserStatus(TUserStatus.cusOnline);

            // Create a user object to represent the caller destination)
            User sinkUser = _skypeObject.get_User(_sinkUsername);
            if (sinkUser == null)
            {
                Console.WriteLine("Couldn't find the Skype user " + _sinkUsername);
                return;
            }

            if (CallAttaching != null)
                CallAttaching(true);

            // Place the call!
            _skypeCall = _skypeObject.PlaceCall(sinkUser.Handle, null, null, null);            
        }

        public void StopCall()
        {
            if (_skypeCall != null)
                _skypeCall.Finish();
        }

        void _skypeObject_CallStatus(Call pCall, TCallStatus status)
        {
            // Keep the stupid client from popping up
            _skypeObject.Client.Minimize();

            if (_skypeCall == null && status == TCallStatus.clsRinging)
            {
                _skypeCall = pCall;
                _skypeCall.Answer();
            }
            else if (pCall.Id == _skypeCall.Id && AudioStatusChanged != null)
            {
                if (status == TCallStatus.clsInProgress)
                {
                    AudioStatusChanged(true);
                    CallAttaching(false);
                }
                else if (status == TCallStatus.clsBusy ||
                         status == TCallStatus.clsCancelled ||
                         status == TCallStatus.clsFailed ||
                         status == TCallStatus.clsFinished ||
                         status == TCallStatus.clsMissed ||
                         status == TCallStatus.clsRefused)
                {
                    AudioStatusChanged(false);
                    CallAttaching(false);
                    _skypeCall = null;
                }
            }
        }

        void _skypeObject_UserStatus(TUserStatus status)
        {
            // If we fell off, then reconnect!
            if (status == TUserStatus.cusOffline)
                Reconnect();
        }

        public bool IsTalking()
        {
            if (_skypeObject == null || _skypeCall == null)
                return false;
            else
                return (_skypeObject.CurrentUserStatus == TUserStatus.cusOnline &&
                        _skypeCall.Status == TCallStatus.clsInProgress);
        }

        // Helper function to make sure the client is up and running
        private void Reconnect()
        {
            while (!_skypeObject.Client.IsRunning)
            {
                try
                {
                    _skypeObject.Client.Start(true, true);

                    if (!_skypeObject.Client.IsRunning)
                        Thread.Sleep(1000);
                }
                catch (COMException) { }
            }

            while (((ISkype)_skypeObject).AttachmentStatus != TAttachmentStatus.apiAttachSuccess)
            {
                try
                {
                    _skypeObject.Attach(5, true);

                    if (((ISkype)_skypeObject).AttachmentStatus != TAttachmentStatus.apiAttachSuccess)
                        Thread.Sleep(1000);
                }
                catch (COMException) { }
            }
        }
    }
}
